unit LBCheck;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls , Menus;

const
  Margin = 2;

type
  TCheckEvent = procedure ( Sender : TObject ; Index : Integer ) of object;

  TCheckListBox = class(TCustomListBox)
  private
    FChecked      : TList;
    FBitmap 			: TBitmap;			{ The standard Windows checkbox bitmaps }
    FAuthor			  : string;				{ Me 																		}
    FOnCheck 		  : TCheckEvent;	{ OnCheck event property 								}
    FOnUnCheck    : TCheckEvent;	{ OnUnCheck event property 							}
    FUserCheck    : Boolean;
    FClickInBox   : Boolean;

    { Read function for Checked property }
    function  IsChecked( Index : Integer ) : Boolean;
    { Write fucntion for Checked property , sets Checked[ Index ] := AChecked }
    procedure Check( Index : Integer ; AChecked : Boolean );
    { Draws the checkbox for the given item in the appropriate state }
    procedure DrawCheckBox( Index : Integer ; State: TOwnerDrawState );
    procedure SetAuthor( Author : string );
    procedure SetBitmap( val : TBitmap);
    function CheckBoxRect( Index : Integer ) : TRect;
    function GetAllChecked : Boolean;
    function GetNoneChecked : Boolean;
 protected
      { Overriden owner draw method }
      procedure DrawItem( Index: Integer; ARect: TRect; State: TOwnerDrawState);override;
      { Overriden mouse down event }
      procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
      { Protected handlers for OnCheck and OnUncheck events }
      procedure CheckEvent( Index : Integer ); virtual;
      procedure UnCheckEvent( Index : Integer ); virtual;
 published
     {	Properties not published 	}
     {   Style 			: set to lbOwnerDrawFixed by the constructor	}
     {   OnDrawItem 	: used to draw check boxes										}

   	{ Properties Added 					}
     property Author 		: string read FAuthor write SetAuthor;
     property OnCheck 		: TCheckEvent read FOnCheck write FOnCheck;
     property OnUncheck 	: TCheckEvent read FOnUncheck write FOnUnCheck;
     property Bitmap : TBitmap read FBitmap write SetBitmap;
     property AllowUserCheck : Boolean read FUserCheck write FUserCheck default True;
     property MustClickInBox : Boolean read FClickInBox write FClickInBox default True;
     { Original properties 			}
     property Align; property BorderStyle; property Color; property Columns;
     property Ctl3D; property Cursor; property DragCursor; property DragMode;
     property Enabled; property ExtendedSelect; property Font; property Height;
     property HelpContext; property Hint; property IntegralHeight; property ItemHeight;
     property Items; property Left; property MultiSelect; property Name;
     property OnClick; property OnDblClick; property OnDragDrop; property OnDragOver;
     property OnEndDrag; property OnEnter; property OnExit; property OnKeyDown;
     property OnKeyPress; property OnKeyUp; property OnMeasureItem; property OnMouseDown;
     property OnMouseMove; property OnMouseUp; property ParentColor; property ParentCtl3D;
     property ParentFont; property ParentShowHint; property PopupMenu; property ShowHint;
     property Sorted; property TabOrder; property TabStop; property Tag; property Top;
     property Visible; property Width;
		public
   	  { Overriden constructor and destructor }
			constructor Create( AParent : TComponent ); override;
      destructor Destroy; override;
      { Checked property. When True , the item is checked }
   	  property Checked[ Index : Integer ] : Boolean read 	IsChecked write Check;
      { Sets the Checked property to AChecked for all selected items }
      property AllChecked : Boolean read GetAllChecked;
      property NoneChecked : Boolean read GetNoneChecked;
      procedure CheckSelection( AChecked : Boolean );
      { Toggles the check of the given item }
      procedure Toggle( Index : Integer );
      { Sets the Checked property to AChecked for all items in the list box }
      procedure CheckAll( AChecked : Boolean );
  end;

procedure Register;

implementation

{------------------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Samples', [TCheckListBox]);
end;
{------------------------------------------------------------------------------}
{ PUBLIC , OVERRIDE																														}
{ Constructor for TCheckListBox																								}
constructor TCheckListBox.Create( AParent : TComponent );
begin
 inherited Create( AParent );
 FChecked := TList.Create;
 Style := lbOwnerDrawFixed;
 FBitmap :=TBitmap.Create;
 FUserCheck := True;
 FClickInBox := True;
 FAuthor := 'Pablo Pissanetzky';
end;
{------------------------------------------------------------------------------}
{ PUBLIC	, OVERRIDE																														}
{ Constructor for TCheckListBox																								}
destructor TCheckListBox.Destroy;
begin
  FBitmap.Free;
  FChecked.Free;
  inherited Destroy;
end;

procedure TCheckListBox.SetBitmap( val : TBitmap);
begin
  FBitmap.Assign(val);
  Refresh;
end;

function TcheckListBox.CheckBoxRect( Index : Integer ) : TRect;
begin
  Result := ItemRect( Index );
  with Result do
    begin
      Inc( Left , Margin );
      Top := Top + ( ( Bottom - Top ) - FBitmap.Height ) div 2;
      Right := Left + FBitmap.Width div 2 ;
      Bottom := Top + FBitmap.Height;
    end;
end;

{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Draws the check box for the item at Index in the given state 								}
procedure TCheckListBox.DrawCheckBox( Index : Integer ; State: TOwnerDrawState );
var
	ARect : TRect;
  CheckedSource 	: TRect;
  UncheckedSource : TRect;
begin
  with CheckedSource do
  begin
    Left:=FBitmap.Width div 2;
    Top:=0;
    Right:=FBitmap.Width;
    Bottom:=FBitmap.Height;
  end;
  with UncheckedSource do
  begin
    Left:=0;
    Top:=0;
    Right:=FBitmap.Width div 2 ;
    Bottom:=FBitmap.Height;
  end;

  ARect := CheckBoxRect( Index );

	with Canvas do
 	begin
     if ( odSelected in State ) then { if this item is selected, draw in reverse }
       CopyMode := cmNotSrcCopy
     else
       CopyMode := cmSrcCopy;

     if ( Checked[ Index ] ) then { copy the appropriate bitmap }
      CopyRect( ARect , FBitmap.Canvas , CheckedSource )
     else
      CopyRect( ARect , FBitmap.Canvas , UnCheckedSource );
   end;
end;
{------------------------------------------------------------------------------}
{ PROTECTED , OVERRIDE																													}
{ Clears the rectangle, draws the check box and draws the item text 						}
procedure TCheckListBox.DrawItem( Index: Integer; ARect: TRect; State: TOwnerDrawState);
begin
	with Canvas do
 	begin
   	FillRect( ARect ); 							{ Clear rectangle }
    DrawCheckBox( Index , State ); 	{ Draw the check box }
    with ARect do
      begin
        Left := ARect.Left + FBitmap.Width div 2 + ( Margin * 2 );
        Top := Top + ( ( Bottom - Top ) - TextHeight( Items[ Index ] ) ) div 2;
        TextOut( Left , Top , Items[ Index ] ); { Draw the text }
      end;
  end;
end;
{------------------------------------------------------------------------------}
{ PROTECTED , OVERRIDE 																												}
{ Calls inherited MouseUp ( which calls user's mouse up )									}
{ If the user has clicked inside a check box , toggle the checkbox and draw 		}
{ the focus rectangle.																													}
procedure TCheckListBox.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 Index 		: Integer;
 Rect      : TRect;
begin
 inherited MouseUp( Button , Shift , X , Y );	{ call inherited method }
 if not FUserCheck then exit;       { Don't allow the user to change the check state }
	if ( Button = mbLeft ) then
 	begin
 		Index := ItemAtPos( Point( X , Y ) , True );{ Find the index of the item clicked }
 		if ( Index > -1 ) then											{ If it is a valid item }
   		begin
        if MustClickInBox then
          Rect := CheckBoxRect( Index )
        else
     		  Rect := ItemRect( Index );							{ Get the rectangle for the item }

         if PtInRect( Rect , Point( X , Y ) ) then { if the user clicked inside the check box }
         	begin
     				Toggle( Index );												{ Toggle the item's checked state }
     				Canvas.DrawFocusRect( ItemRect( Index ) );	{ draw focus rectangle }
           end;
   		end;
   end;
end;
{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Returns True if the item at Index is Checked, otherwise returns False 				}
function TCheckListBox.IsChecked( Index : Integer ) : Boolean;
begin
  Result := FChecked.IndexOf( Pointer( Index ) ) > -1;
end;
{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Sets the item at index to be checked if AChecked is True 										}
{ or unchecked if AChecked is False																						}
{ If the item was not already in the given state, it redraws the check box			}
{ and generates either an OnCheck or OnUnCheck event 													}
procedure TCheckListBox.Check( Index : Integer ; AChecked : Boolean );
var
	State : TOwnerDrawState;
begin
  if IsChecked( Index ) <> AChecked then
    begin
      if not AChecked then
        FChecked.Delete( FChecked.IndexOf( Pointer( Index ) ) )
      else
        FChecked.Add( Pointer( Index ) );

 			if ( Selected[ Index ] ) then   			{ if the item is selected, it must be drawn accordingly }
 				State := [ odSelected ]
       else
       	State := [];
       DrawCheckBox( Index , State );				{ draw the check box }
       if ( AChecked ) then                  { generate appropriate event }
       	CheckEvent( Index )
       else
       	UnCheckEvent( Index );
     end;
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Sets all selected items to checked if AChecked is True, unchecked otherwise	}
procedure TCheckListBox.CheckSelection( AChecked : Boolean );
var
	Index : Integer;
begin
	if ( SelCount > 0 ) then
 	  for Index := 0 to Items.Count - 1 do
      if ( Selected[ Index ] ) then
     	  Checked[ Index ] := AChecked;
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Toggles the state of the item at the given index 														}
procedure TCheckListBox.Toggle( Index : Integer );
begin
	Checked[ Index ] := not Checked[ Index ];
end;
{------------------------------------------------------------------------------}
{ PROTECTED , VIRTUAL  																												}
{ Calls user's OnCheck event handler																						}
procedure TCheckListBox.CheckEvent( Index : Integer );
begin
	if ( Assigned( FOnCheck ) ) then
 	  OnCheck( Self , Index );
end;
{------------------------------------------------------------------------------}
{ PROTECTED , VIRTUAL 																													}
{ Calls user's OnUnChecked event handler																				}
procedure TCheckListBox.UnCheckEvent( Index : Integer );
begin
	if ( Assigned( FOnUncheck ) ) then
 	  OnUnCheck( Self , Index );
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Checks all items in the list box if AChecked is True, unchecks them otherwise}
procedure TCheckListBox.CheckAll( AChecked : Boolean );
var
	Index : Integer;
begin
	for Index := 0 to Items.Count - 1 do
 	  Checked[ Index ] := AChecked;
end;
{------------------------------------------------------------------------------}
function TCheckListBox.GetAllChecked : Boolean;
begin
  Result := FChecked.Count = Items.Count;
end;
function TCheckListBox.GetNoneChecked : Boolean;
begin
  Result := FChecked.Count = 0;
end;
{------------------------------------------------------------------------------}
procedure TCheckListBox.SetAuthor( Author : string );
begin
end;
{------------------------------------------------------------------------------}
end.
